#!/usr/bin/perl -wT
#
# account/umod.cgi
#
# Program to change a users full name and/or password.
#
# Copyright 1998 Corel Computer Corporation
#
# This is open-source software (see www.opensource.org).  Open Source(R)
# is a Registered Certification Mark of Software in the Public Interest.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 1,
# or, (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to the
# Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

use strict;
use CGI;
use CGI::Carp qw/fatalsToBrowser/;
my $q = new CGI;
require '../layout.pl';
require '../common.pl';

BEGIN { $ENV{'PATH'} = '/bin:/usr/bin'; }

# Any errors found during processing added here:

my @errors;

# Begin validation sequence

push @errors, "Remote username could not be determined."
unless defined $ENV{'REMOTE_USER'};

my @user = getpwnam($ENV{'REMOTE_USER'}) or
push @errors, "User is not authorized for this service.";

push @errors, "You must enter your real name."
unless defined $q->param('nam') and $q->param('nam') ne '';
my $name = $q->param('nam');

push @errors, "The name must contain only alphanumeric characters."
if $name =~ /[^-_. a-zA-Z0-9]/;

my $newpass;
if (defined $q->param('pw1') and defined $q->param('pw2')) {
  push @errors, "Passwords do not match, please try again."
  unless $q->param('pw1') eq $q->param('pw2');
  $newpass = $q->param('pw1');
}

# Check for errors, abort if any found.

if (@errors) {
  print start_webfront("Confirmation","user",3,-1);
  print CGI::h1((($#errors > 0) ? "Errors have" : "An error has")." been detected");
  print CGI::ul(CGI::li(\@errors));
  print CGI::p(CGI::i('Go <A HREF="main.cgi">back</A> and try again.'));
  print end_webfront();
  exit(1);
} 

# Do regular processing
if ($newpass ne '') {
  # Generate a random salt string for encryption
  my @chars = ('a'..'z', 'A'..'Z', '.', '/');
  my $salt = $chars[rand $#chars] . $chars[rand $#chars];
  # Encrypt the password and substitute it
  $user[1] = crypt $newpass, $salt;
}
$user[5] = $name;
my $replacement = join(":", @user[0,1,2,3,5,7,8]) . "\n";

# Now scan/replace password file entry
umask(022);
open IN, "/etc/passwd" or die "Can't read passwd\n";
open OUT, ">/etc/passwd.new" or die "Can't write passwd\n";
while (my $line = <IN>) {
  my @line = split /:/, $line;
  # Only change lines with matching login, and UID>100
  if ($line[0] eq $user[0] and $line[2] >= 100) {
    print OUT $replacement;
  } else {
    print OUT $line;
  }
}
close OUT;
close IN;
system "mv /etc/passwd.new /etc/passwd";
system "/usr/bin/htpasswd.sh";
print start_webfront("Confirmation","user",3,-1);
print CGI::h1("Operation was successful.");
print CGI::p('Click <A HREF="main.cgi">here</A> to return.');
print end_webfront();

